#ML estimation
#example: unfair coin flipped 20 times
flips<-sample(x=c(rep(x="face", times=15), rep("pile", 5)), replace=FALSE)
flips

table(flips)

#p = 0.5
dbinom(x=15, size=20, prob=0.5)

#p = 0.6
dbinom(x=15, size=20, prob=0.6)

#p = 0.9
dbinom(x=15, size=20, prob=0.9)

x<-15
n<-20
theta<-seq(from=0, to=1, by=0.01)
Like<-choose(n=20, k=x)*(theta^x)*((1-theta)^(n-x))
L<-dbinom(x=15, size=20, prob=theta)
LL<-dbinom(x=15, size=20, prob=theta, log=TRUE)

#theta<-15/20
#plot likelihood curve
plot(Like~theta, main="Fonction du likelihood", xlab="Probabilit d'observer face", ylab="Likelihood", type="l")
#maximum value of likelihood
max(Like)
which(Like==max(Like))
theta[76]
abline(v=0.75, col="magenta", lty="dashed")

Like_norm<-Like/max(Like)  #valeurs du likelihood normalis pour avoir un maximum de 1


#plot log-likelihood curve
plot(LL~theta, main="Fonction du log-likelihood", xlab="Probabilit d'observer face",
ylab="log-likelihood", type="l")
#maximum value of likelihood
max(LL)
which(Like==max(Like))
theta[76]
abline(v=0.75, col="magenta", lty="dashed")




###################################
#occupancy analyses examples
#import data
detect <- read.table(file = "L:/home/mazerolm/Documents/Stats consulting/Occupancy Workshops/Detect_frog.txt",
header = TRUE)

nsites <- length(detect[,1])

#create character vector of detection history
hist<-rep(x = NA, times = nsites)                   #create a vector to hold the detection histories
for (i in 1:nsites) {                           #collapse each visit into a single column to obtain a detection history
  hist[i]<-paste(detect[i, 2:6], sep="", collapse="")
  }

#summarize number of each detection history
table(hist)

#summarize results into a pivot table and convert into a data frame
freqs <- as.data.frame(table(hist, deparse.level=2))



#assemble the likelihood by combining the probability statements
#model with constant occupancy and detectability
site.occ <- function(p) {
psi <- p[1]
p <- p[2]
(16*log((psi*(1-p)*(1-p)*(1-p)*(1-p)*(1-p) + (1 - psi)))) +
(3*log((psi*(1-p)*(1-p)*(1-p)*(1-p)*(p)))) +
(3*log((psi*(1-p)*(1-p)*(1-p)*(p)*(1-p)))) +
(log((psi*(1-p)*(1-p)*(1-p)*(p)*(p)))) +
(2*log((psi*(1-p)*(1-p)*(p)*(1-p)*(1-p)))) +
(log((psi*(1-p)*(1-p)*(p)*(1-p)*(p)))) +
(3*log((psi*(1-p)*(1-p)*(p)*(p)*(1-p)))) +
(3*log((psi*(1-p)*(1-p)*(p)*(p)*(p)))) +
(log((psi*(1-p)*(p)*(p)*(1-p)*(1-p)))) +
(log((psi*(1-p)*(p)*(p)*(p)*(1-p))))
}

neg.LL <- function(p) {-1*site.occ(p)} #function nlm minimizes likelihood
#estimate parameters
hold <- nlm(f = neg.LL, p = c(0.5, 0.5), hessian = TRUE)
hold2<-optim(neg.LL, p=c(0.5,0.5),hessian=T, method="BFGS")
#vcov matrix
var_cov<-solve(hold$hessian)
#SE's of parameters
sqrt(diag(var_cov))


#model with constant occupancy and detectability, but specified as logistic link
#more efficient numerically
site_occ<-function(p){
psi<-(exp(p[1]))/(1+exp(p[1]))
p<-exp(p[2])/(1+exp(p[2]))
(16*log((psi*(1-p)*(1-p)*(1-p)*(1-p)*(1-p) + (1 - psi)))) +
(3*log((psi*(1-p)*(1-p)*(1-p)*(1-p)*(p)))) +
(3*log((psi*(1-p)*(1-p)*(1-p)*(p)*(1-p)))) +
(log((psi*(1-p)*(1-p)*(1-p)*(p)*(p)))) +
(2*log((psi*(1-p)*(1-p)*(p)*(1-p)*(1-p)))) +
(log((psi*(1-p)*(1-p)*(p)*(1-p)*(p)))) +
(3*log((psi*(1-p)*(1-p)*(p)*(p)*(1-p)))) +
(3*log((psi*(1-p)*(1-p)*(p)*(p)*(p)))) +
(log((psi*(1-p)*(p)*(p)*(1-p)*(1-p)))) +
(log((psi*(1-p)*(p)*(p)*(p)*(1-p))))
}

neg.LL <- function(p) {-1*site_occ(p)} #function nlm minimizes likelihood
#estimate parameters on logit scale
hold <- nlm(f = neg.LL, p = c(0, 0), hessian = TRUE)

#parameter estimates on logit scale
hold$estimate

#vcov matrix of parameter estimates on logit scale
SE<-sqrt(diag(solve(hold$hessian)))

#back-transform estimates
expit <- function(mu) {
exp(mu)/(1+exp(mu))
}
expit(hold$est)

#95% CI
CL95 <- hold$estimate[1]+1.96*c(-SE[1], SE[1])
expit(CL95)




#FROM PRESENCE
#compute logit(psi) for first site based on model psi(Distpond) p(.)
logit.psi <- 0.6782 + (-0.0351*-2.6114)
expit(logit.psi)

logit.p <- -0.9060
expit(logit.p)


#set up vector of model names
Modname=c("psi(Distpond),p(Airtemp)","psi(.),p(Airtemp)","psi(Distpond),p(.)","psi(.)p(.)")
#-2*LL
min2.LL <- c(132.4933, 135.9888, 155.0826, 158.8044)
#number of parameters
Ks <- c(4, 3, 3, 2)

#Model selection based on AICc
n <- 34 #number of sites
Results <- data.frame(Modname)
Results$LL <- min2.LL/-2
Results$K <- Ks
Results$AICc <- -2*Results$LL+2*Results$K*(n/(n-Results$K-1))
Results$Delta_AICc <- Results$AICc-min(Results$AICc)
Results$ModelLik <- exp(-0.5*Results$Delta_AICc)
Results$AICcWt <- Results$ModelLik/sum(Results$ModelLik)
AICctable <- Results[rev(order(Results$AICcWt)), ]
#round results to 2 digits after decimal point for nicer printing
AICc.round <- cbind(AICctable$Modname, round(AICctable[,2:7], digits=2))

#evidence ratio of best vs 2nd best model
evidence.ratio <- AICctable[1, 7]/AICctable[2, 7]

#prediction for Distpond = 27.1 m (corresponds to first site)
#corresponds to Distpond_cent = 27.1 - 29.7 (29.7 is the mean used to center Distpond)
#Distpond_cent = -2.6 (this is the first value in Distpond_cent)

#predicted values based on models in the same order as AICctable
preds <- c(0.6386, 0.6165, 0.6835, 0.6434)

#SE's of predicted values computed from delta method
se.preds <- c(0.1200, 0.1087, 0.1449, 0.1187)

#compute model-averaged predicted values
modavg.preds <- sum(AICctable$AICcWt*preds)

#compute unconditional SE based on revised equation (6.12 of Burnham and Anderson)
uncond.se.new <- sqrt(sum(AICctable$AICcWt*se.preds^2 + (preds - modavg.preds)^2))


#compute unconditional SE based on old equation (4.9 of Burnham and Anderson 2002)
uncond.se.old <- sum(AICctable$AICcWt*sqrt(se.preds^2 + (preds - modavg.preds)^2))



#compute model-averaged estimate of Airtemp
#recompute AICc table for models including Airtemp
n <- 34 #number of sites
Results <- data.frame(Modname=Modname[1:2])
Results$LL <- min2.LL[1:2]/-2
Results$K <- Ks[1:2]
Results$AICc <- -2*Results$LL+2*Results$K*(n/(n-Results$K-1))
Results$Delta_AICc <- Results$AICc-min(Results$AICc)
Results$ModelLik <- exp(-0.5*Results$Delta_AICc)
Results$AICcWt <- Results$ModelLik/sum(Results$ModelLik)
AICctable <- Results[rev(order(Results$AICcWt)), ]
#round results to 2 digits after decimal point for nicer printing
AICc.round <- cbind(AICctable$Modname, round(AICctable[,2:7], digits=2))

#estimates and SE's of Airtemp
airtemp.est <- c(0.212989, 0.214914)
airtemp.se <- c(0.056029, 0.056167)

#model-averaged estimate
modavg.est <- sum(AICctable$AICcWt*airtemp.est)

#compute unconditional SE based on revised equation (6.12 of Burnham and Anderson)
uncond.se.new <- sqrt(sum(AICctable$AICcWt*airtemp.se^2 + (airtemp.est - modavg.est)^2))

#95% unconditonal CI
modavg.est+1.96*c(-uncond.se.new, uncond.se.new)


#time-dependent detection
-20.36
exp(-20.36)/(1+exp(-20.36))